#include <GDD/ddg.h>

int main(int argc, char *argv[]) {
  // define several operations
  GDD_Opcode *i1 =  GDD_opcode_create("inst_1", 1, 0);
  GDD_opcode_extend_delta_w(i1, "GR", 0);

  GDD_Opcode *i2 =  GDD_opcode_create("inst_2", 1, 0);
  GDD_opcode_extend_delta_w(i2, "GR", 0);

  GDD_Opcode *i3 =  GDD_opcode_create("inst_3", 1, 0);
  GDD_opcode_extend_delta_w(i3, "GR", 0);

  GDD_Opcode *i4 =  GDD_opcode_create("inst_4", 1, 0);
  GDD_opcode_extend_delta_w(i4, "GR", 0);

  GDD_Opcode *i5 =  GDD_opcode_create("inst_5", 1, 0);
  GDD_opcode_extend_delta_w(i5, "GR", 0);

  GDD_Opcode *i6 =  GDD_opcode_create("inst_6", 1, 0);
  GDD_opcode_extend_delta_w(i6, "BR", 0);

  GDD_Opcode *i7 =  GDD_opcode_create("inst_7", 1, 0);

  // build the architecture description
  GDD_Arch *arch = GDD_arch_create();
  GDD_arch_add_regtype(arch, "GR", 61);
  GDD_arch_add_regtype(arch, "BR", 8);
  GDD_arch_add_opcode(arch, i1);
  GDD_arch_add_opcode(arch, i2);
  GDD_arch_add_opcode(arch, i3);
  GDD_arch_add_opcode(arch, i4);
  GDD_arch_add_opcode(arch, i5);
  GDD_arch_add_opcode(arch, i6);
  GDD_arch_add_opcode(arch, i7);

  // build a GDD
  SCEDA_Graph *g = SCEDA_graph_create((SCEDA_delete_fun)GDD_node_label_delete, (SCEDA_delete_fun)GDD_edge_label_delete); 

  SCEDA_Vertex *v1 = SCEDA_graph_add_vertex(g, GDD_node_label_create(1, i1));
  SCEDA_Vertex *v2 = SCEDA_graph_add_vertex(g, GDD_node_label_create(2, i2));
  SCEDA_Vertex *v3 = SCEDA_graph_add_vertex(g, GDD_node_label_create(3, i3));
  SCEDA_Vertex *v4 = SCEDA_graph_add_vertex(g, GDD_node_label_create(4, i4));
  SCEDA_Vertex *v5 = SCEDA_graph_add_vertex(g, GDD_node_label_create(5, i5));
  SCEDA_Vertex *v6 = SCEDA_graph_add_vertex(g, GDD_node_label_create(6, i6));
  SCEDA_Vertex *v7 = SCEDA_graph_add_vertex(g, GDD_node_label_create(7, i7));

  SCEDA_graph_add_edge(g, v1, v2, GDD_edge_label_create("GR", "flowdep_reg", 3, 0));
  SCEDA_graph_add_edge(g, v1, v7, GDD_edge_label_create(NULL, "serial", 0, 0));
  SCEDA_graph_add_edge(g, v2, v3, GDD_edge_label_create("GR", "flowdep_reg", 3, 0));
  SCEDA_graph_add_edge(g, v3, v3, GDD_edge_label_create("GR", "flowdep_reg", 1, 1));
  SCEDA_graph_add_edge(g, v3, v7, GDD_edge_label_create("GR", "flowdep_reg", 1, 0));      
  SCEDA_graph_add_edge(g, v4, v4, GDD_edge_label_create("GR", "flowdep_reg", 1, 1));
  SCEDA_graph_add_edge(g, v4, v6, GDD_edge_label_create("GR", "flowdep_reg", 1, 0));
  SCEDA_graph_add_edge(g, v5, v1, GDD_edge_label_create(NULL, "serial", 1, 4));
  SCEDA_graph_add_edge(g, v5, v5, GDD_edge_label_create("GR", "flowdep_reg", 1, 1));
  SCEDA_graph_add_edge(g, v6, v7, GDD_edge_label_create("BR", "flowdep_reg", 3, 0));

  // print the values of type GR and their killers
  SCEDA_HashSet *v_gr = GDD_get_values_of_type(g, "GR");

  // iterate over the GR values
  SCEDA_HashSetIterator values;
  SCEDA_hashset_iterator_init(v_gr, &values);
  while(SCEDA_hashset_iterator_has_next(&values)) {
    // print the mnemonic of the value
    SCEDA_Vertex *v = SCEDA_hashset_iterator_next(&values);
    fprintf(stdout,"%s is a value of type GR\n", GDD_vertex_mnemonic(v));
    
    // compute the set of consumers
    SCEDA_HashSet *cons_v = GDD_consumers_of_type(v, "GR");

    // iterate over the set of consumers
    SCEDA_HashSetIterator consumers;
    SCEDA_hashset_iterator_init(cons_v, &consumers);
    while(SCEDA_hashset_iterator_has_next(&consumers)) {
      // print the mnemonic of the consumer
      SCEDA_Vertex *w = SCEDA_hashset_iterator_next(&consumers);
      fprintf(stdout,"%s consumes it\n", GDD_vertex_mnemonic(w));
    }
    SCEDA_hashset_iterator_cleanup(&consumers);

    // delete the set of consumers
    SCEDA_hashset_delete(cons_v);
  }
  SCEDA_hashset_iterator_cleanup(&values);

  // delete the set of values
  SCEDA_hashset_delete(v_gr);

  // check whether the GDD is lexicographic positive
  SCEDA_List *cycle;
  if(GDD_is_lexicographic_positive(g, &cycle)) {
    // yes!
    fprintf(stdout,"GDD is lexicographic positive\n");

    // compute the MII ratio & cycle
    int MIInum, MIIden;
    if(GDD_MII_ratio_cycle(g, &MIInum, &MIIden, &cycle) == 0) {
      // just computed
      fprintf(stdout,"MII ratio is %d/%d\n", MIInum, MIIden);

      // print the cycle
      SCEDA_ListIterator edges;
      SCEDA_list_iterator_init(cycle, &edges);
      while(SCEDA_list_iterator_has_next(&edges)) {
	SCEDA_Edge *e = SCEDA_list_iterator_next(&edges);
	SCEDA_Vertex *vs = SCEDA_edge_source(e);
	SCEDA_Vertex *vt = SCEDA_edge_target(e);
	fprintf(stdout,"%s -(%d,%d) -> %s\n", GDD_vertex_mnemonic(vs), GDD_edge_latency(e), GDD_edge_distance(e), GDD_vertex_mnemonic(vt));
      }
      SCEDA_list_iterator_cleanup(&edges);

      // do not forget to delete the list  (allocated by GDD_MII_ratio_cycle)
      SCEDA_list_delete(cycle);
    } else {
      // here, an error just happened
      fprintf(stdout,"unable to compute MII ratio\n");
    }
  } else {
    // not lexicographic positive
    fprintf(stdout,"GDD is not lexicographic positive\n");

    // do not forget to delete the list (allocated by GDD_is_lexicographic_positive)
    SCEDA_list_delete(cycle);
  }

  // free memory
  SCEDA_graph_delete(g); // delete g
  GDD_arch_delete(arch);     // and the architecture

  return 0;
}
